<?php

namespace MyApp\Core;

use PDOException;
use ReflectionClass;
use ReflectionProperty;
use PDO;

class Model
{
    /**
     * @var array
     */
    private $internalTableMap;
    /**
     * @var \App
     */
    protected $app;

    public function __construct(?array $tableMap = null)
    {
        global $app;
        $this->app = $app;

        if ( ! is_array($tableMap)) {
            $this->setInternalTableMap();
        }

        if ('db' === $this->getModelType()) {
            $this->initDb();
        }
    }

    private function initDb(): void
    {
        if (is_null($this->app->db)) {
            try {
                $this->app->db = new PDO(
                    'mysql:host=mysql;dbname='
                    . $this->app->config->getDbName()
                    . ';port='
                    . $this->app->config->getDbPort(),
                    $this->app->config->getDbUser(),
                    $this->app->config->getDbPassword(),
                    array(
                        PDO::MYSQL_ATTR_INIT_COMMAND => 'SET NAMES UTF8',
                        PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_OBJ,
                        PDO::ATTR_ERRMODE            => true,
                        PDO::ATTR_PERSISTENT         => true,
                    ));
            } catch (PDOException $e) {
                echo 'Подключение не удалось: ' . $e->getMessage();
            }

        }
    }

    private function setInternalTableMap()
    {
        /* @var \App $app */
        global $app;

        $tableMap = ['type' => 'db'];

        $refClass = new ReflectionClass(get_called_class());
        $tags     = $app->parseTagsFromComment($refClass->getDocComment(),
            'TableName');
        if ($tags) {
            if (array_key_exists('TableName', $tags)) {
                $tableMap['name'] = $tags['TableName'];
            }

            $properties = $refClass->getProperties(ReflectionProperty::IS_PRIVATE);
            foreach ($properties as $property) {
                $propertyTags = $app->parseTagsFromComment($property->getDocComment(),
                    'ColumnName|ColumnOption|ColumnType');

                if ($propertyTags) {
                    if (array_key_exists('ColumnName', $propertyTags)) {
                        $tableMap['fields'][$property->name] = $propertyTags['ColumnName'];
                        if (array_key_exists('ColumnOption', $propertyTags)) {
                            if ('id' === $propertyTags['ColumnOption']) {
                                $tableMap['id'] = $property->name;
                            }
                        }
                        if (array_key_exists('ColumnType', $propertyTags)) {
                            $tableMap['types'][$propertyTags['ColumnName']] = $propertyTags['ColumnType'];
                        }
                    }
                }
            }
            $this->internalTableMap = array_key_exists('id',
                $tableMap) ? $tableMap : ['type' => 'other'];
        }
        if ( ! is_array($this->internalTableMap)) {
            $this->internalTableMap = ['type' => 'other'];
        }
    }

    public static function find()
    {
    }

    public static function findOne()
    {
    }

    /**
     * @return array
     */
    public function getInternalTableMap(): ?array
    {
        return $this->internalTableMap;
    }

    public function getModelType(): string
    {
        return $this->internalTableMap['type'];
    }
}
